==> iRRAM_2003_03.tar.gz: iRRAM_2003_03/src/pi_ln2.cc <==
#include "stdio.h"
#include "stdlib.h"

#include "iRRAM_core.h"

/*****************************************************************/
/*                     ln(2)=0.6931471805....                    */
/*   Usage: ln2()                                                */
/*   Application: range reduction, AGM methods for logarithms, ..*/ 
/*****************************************************************/

REAL ln2_val=1;
long ln2_err=0;


REAL ln2_approx (long prec){
  long N=100-prec/2;
  REAL a=1,b=scale(REAL(1),N),c;
  do {
    c=a*b;
    a=2*c/(a+b);
    b=sqrt(c);
//  } while ( !bound(a-b,-N) );
//  c= a*b/(a+b)/(N+2);
  } while ( !bound(a-b,-2*N) );
  c= a/(2*N+4);
  return c;
};

REAL ln2(){
   if (ln2_err >  REAL_actual_prec || ln2_err == 0) {
     ln2_err =  REAL_actual_prec;
     ln2_val = pi()*limit(ln2_approx);
    }
   return ln2_val;
};


/*****************************************************************/
/*                     pi = 3.141592653...                       */
/*   Usage: pi()                                                 */
/*   Application: range reduction, ...                           */ 
/*****************************************************************/


REAL pi_val=3;
long pi_err=0;

REAL pi_approx_MACHIN (long prec){
   REAL z1=REAL(1)/REAL(5);
   REAL z2=REAL(1)/REAL(239);
   REAL y,z;
   long i;
  
   i=3;
   y=z1;
   z=y;
   while ( !bound(z,prec-5) ) {
     y=-y/(5*5);
     z=y/i;
     z1=z1+z;
     i+=2;
   }
   i=3;
   y=z2;
   z=y;
   while ( !bound(z,prec-3) ) {
     y=-y/(239*239);
     z=y/i;
     z2=z2+z;
     i+=2;
   }
   return 4*(4*z1-z2);
};

REAL pi_approx_AGM (long prec){
   REAL a= 1;
   REAL b= 1/sqrt(REAL(2));
   REAL t= 1/REAL(4);
   REAL pi_l,pi_u;
   long j=1;
   REAL y;
   do {
     y=a;
     a=(a+b)/2;
     b=sqrt(b*y);
     t=t-j*power(a-y,2);
     j=2*j; 
   } while ( ! bound(a-b, prec));
   return a*a/t;
};

REAL pi_inv_approx_BORWEIN (long prec){
   REAL a= 6-4*sqrt(REAL(2));
   REAL y= sqrt(REAL(2))-1;
   REAL s=2,t,aold;
   long k=1;
   do {
     aold=a;
     t=sqrt(sqrt(1-power(y,4)));
     y=(1-t)/(1+t);
     s=s*4;
     a=a*power(1+y,4)-s*y*(1+y+y*y);
     k=2*k; 
   } while (  !bound (a-aold,prec/4 )) ;
   return a;
};

REAL pi (){
   if (pi_err >  REAL_actual_prec || pi_err == 0) {
     pi_err =  REAL_actual_prec;
//   pi_val = limit(pi_approx_MACHIN);
//   pi_val = limit(pi_approx_AGM);
     pi_val = 1/limit(pi_inv_approx_BORWEIN);
    }
   return pi_val;
};

==> iRRAM_2003_08.tar.gz: iRRAM_2003_08/src/pi_ln2.cc <==
#include "stdio.h"
#include "stdlib.h"

#include "iRRAM_core.h"

/*****************************************************************/
/*                     ln(2)=0.6931471805....                    */
/*   Usage: ln2()                                                */
/*   Application: range reduction, AGM methods for logarithms, ..*/ 
/*****************************************************************/

REAL ln2_val=1;
long ln2_err=0;


REAL ln2_approx (long prec){
  long N=100-prec/2;
  REAL a=1,b=scale(REAL(1),N),c;
  do {
    c=a*b;
    a=2*c/(a+b);
    b=sqrt(c);
//  } while ( !bound(a-b,-N) );
//  c= a*b/(a+b)/(N+2);
  } while ( !bound(a-b,-2*N) );
  c= a/(2*N+4);
  return c;
};

REAL ln2(){
   if (ln2_err >  ACTUAL_STACK.actual_prec || ln2_err == 0) {
     ln2_err =  ACTUAL_STACK.actual_prec;
     ln2_val = pi()*limit(ln2_approx);
    }
   return ln2_val;
};


/*****************************************************************/
/*                     pi = 3.141592653...                       */
/*   Usage: pi()                                                 */
/*   Application: range reduction, ...                           */ 
/*****************************************************************/


REAL pi_val=3;
long pi_err=0;

REAL pi_approx_MACHIN (long prec){
   REAL z1=REAL(1)/REAL(5);
   REAL z2=REAL(1)/REAL(239);
   REAL y,z;
   long i;
  
   i=3;
   y=z1;
   z=y;
   while ( !bound(z,prec-5) ) {
     y=-y/(5*5);
     z=y/i;
     z1=z1+z;
     i+=2;
   }
   i=3;
   y=z2;
   z=y;
   while ( !bound(z,prec-3) ) {
     y=-y/(239*239);
     z=y/i;
     z2=z2+z;
     i+=2;
   }
   return 4*(4*z1-z2);
};

REAL pi_approx_AGM (long prec){
   REAL a= 1;
   REAL b= 1/sqrt(REAL(2));
   REAL t= 1/REAL(4);
   REAL pi_l,pi_u;
   long j=1;
   REAL y;
   do {
     y=a;
     a=(a+b)/2;
     b=sqrt(b*y);
     t=t-j*power(a-y,2);
     j=2*j; 
   } while ( ! bound(a-b, prec));
   return a*a/t;
};

REAL pi_inv_approx_BORWEIN (long prec){
   REAL a= 6-4*sqrt(REAL(2));
   REAL y= sqrt(REAL(2))-1;
   REAL s=2,t,aold;
   long k=1;
   do {
     aold=a;
     t=sqrt(sqrt(1-power(y,4)));
     y=(1-t)/(1+t);
     s=s*4;
     a=a*power(1+y,4)-s*y*(1+y+y*y);
     k=2*k; 
   } while (  !bound (a-aold,prec/4 )) ;
   return a;
};

REAL pi (){
   if (pi_err >  ACTUAL_STACK.actual_prec || pi_err == 0) {
     pi_err =  ACTUAL_STACK.actual_prec;
//   pi_val = limit(pi_approx_MACHIN);
//   pi_val = limit(pi_approx_AGM);
     pi_val = 1/limit(pi_inv_approx_BORWEIN);
    }
   return pi_val;
};
